/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSColorSpace;

import dstep.appkit.AppKitDefines;
import dstep.applicationservices.ApplicationServices;
import dstep.foundation.NSCoder;
import dstep.foundation.NSData;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

alias NSInteger NSColorSpaceModel;

enum
{
	NSUnknownColorSpaceModel = -1,
	NSGrayColorSpaceModel,
	NSRGBColorSpaceModel,
	NSCMYKColorSpaceModel,
	NSLABColorSpaceModel,
	NSDeviceNColorSpaceModel,
	NSIndexedColorSpaceModel,
	NSPatternColorSpaceModel
}

class NSColorSpace : NSObject, INSCoding
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}
	
	this (NSCoder aDecoder)
	{
		super(typeof(this).alloc.initWithCoder(aDecoder).objcObject);
	}
	
	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}
	
	typeof(this) initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(typeof(this), "initWithCoder:", NSCoder)(aDecoder);
	}

	NSColorSpace initWithICCProfileData (NSData iccData)
	{
		id result = invokeObjcSelf!(id, "initWithICCProfileData:", NSData)(iccData);
		return result is this.objcObject ? this : (result !is null ? new NSColorSpace(result) : null);
	}

	this (NSData iccData)
	{
		super(NSColorSpace.alloc.initWithICCProfileData(iccData).objcObject);
	}

	NSData ICCProfileData ()
	{
		return invokeObjcSelf!(NSData, "ICCProfileData");
	}

	NSColorSpace initWithColorSyncProfile (void* prof)
	{
		id result = invokeObjcSelf!(id, "initWithColorSyncProfile:", void*)(prof);
		return result is this.objcObject ? this : (result !is null ? new NSColorSpace(result) : null);
	}

	this (void* prof)
	{
		super(NSColorSpace.alloc.initWithColorSyncProfile(prof).objcObject);
	}

	void* colorSyncProfile ()
	{
		return invokeObjcSelf!(void*, "colorSyncProfile");
	}

	NSColorSpace initWithCGColorSpace (CGColorSpaceRef cgColorSpace)
	{
		id result = invokeObjcSelf!(id, "initWithCGColorSpace:", CGColorSpaceRef)(cgColorSpace);
		return result is this.objcObject ? this : (result !is null ? new NSColorSpace(result) : null);
	}

	this (CGColorSpaceRef cgColorSpace)
	{
		super(NSColorSpace.alloc.initWithCGColorSpace(cgColorSpace).objcObject);
	}

	CGColorSpaceRef CGColorSpace ()
	{
		return invokeObjcSelf!(CGColorSpaceRef, "CGColorSpace");
	}

	NSInteger numberOfColorComponents ()
	{
		return invokeObjcSelf!(NSInteger, "numberOfColorComponents");
	}

	int colorSpaceModel ()
	{
		return invokeObjcSelf!(int, "colorSpaceModel");
	}

	NSString localizedName ()
	{
		return invokeObjcSelf!(NSString, "localizedName");
	}

	static NSColorSpace genericRGBColorSpace ()
	{
		return invokeObjcSelfClass!(NSColorSpace, "genericRGBColorSpace");
	}

	static NSColorSpace genericGrayColorSpace ()
	{
		return invokeObjcSelfClass!(NSColorSpace, "genericGrayColorSpace");
	}

	static NSColorSpace genericCMYKColorSpace ()
	{
		return invokeObjcSelfClass!(NSColorSpace, "genericCMYKColorSpace");
	}

	static NSColorSpace deviceRGBColorSpace ()
	{
		return invokeObjcSelfClass!(NSColorSpace, "deviceRGBColorSpace");
	}

	static NSColorSpace deviceGrayColorSpace ()
	{
		return invokeObjcSelfClass!(NSColorSpace, "deviceGrayColorSpace");
	}

	static NSColorSpace deviceCMYKColorSpace ()
	{
		return invokeObjcSelfClass!(NSColorSpace, "deviceCMYKColorSpace");
	}

	static NSColorSpace sRGBColorSpace ()
	{
		return invokeObjcSelfClass!(NSColorSpace, "sRGBColorSpace");
	}

	static NSColorSpace adobeRGB1998ColorSpace ()
	{
		return invokeObjcSelfClass!(NSColorSpace, "adobeRGB1998ColorSpace");
	}

}

