/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSGlyphInfo;

import dstep.appkit.NSFont;
import dstep.foundation.NSCoder;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

typedef NSUInteger NSCharacterCollection;

enum
{
	NSIdentityMappingCharacterCollection = 0,
	NSAdobeCNS1CharacterCollection = 1,
	NSAdobeGB1CharacterCollection = 2,
	NSAdobeJapan1CharacterCollection = 3,
	NSAdobeJapan2CharacterCollection = 4,
	NSAdobeKorea1CharacterCollection = 5
}

class NSGlyphInfo : NSObject, INSCopying, INSCoding
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}

	this (NSCoder aDecoder)
	{
		super(typeof(this).alloc.initWithCoder(aDecoder).objcObject);
	}
	
	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}
	
	typeof(this) initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(typeof(this), "initWithCoder:", NSCoder)(aDecoder);
	}
	
	typeof(this) copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(typeof(this), "copyWithZone:", NSZone*)(zone);
	}
	
	static NSGlyphInfo glyphInfoWithGlyphName (NSString glyphName, NSFont font, NSString theString)
	{
		return invokeObjcSelfClass!(NSGlyphInfo, "glyphInfoWithGlyphName:forFont:baseString:", NSString, NSFont, NSString)(glyphName, font, theString);
	}

	static NSGlyphInfo glyphInfoWithGlyph (uint glyph, NSFont font, NSString theString)
	{
		return invokeObjcSelfClass!(NSGlyphInfo, "glyphInfoWithGlyph:forFont:baseString:", uint, NSFont, NSString)(glyph, font, theString);
	}

	static NSGlyphInfo glyphInfoWithCharacterIdentifier (NSUInteger cid, uint characterCollection, NSString theString)
	{
		return invokeObjcSelfClass!(NSGlyphInfo, "glyphInfoWithCharacterIdentifier:collection:baseString:", NSUInteger, uint, NSString)(cid, characterCollection, theString);
	}

	NSString glyphName ()
	{
		return invokeObjcSelf!(NSString, "glyphName");
	}

	NSUInteger characterIdentifier ()
	{
		return invokeObjcSelf!(NSUInteger, "characterIdentifier");
	}

	uint characterCollection ()
	{
		return invokeObjcSelf!(uint, "characterCollection");
	}

}

