/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSTableColumn;

import dstep.appkit.NSCell;
import dstep.appkit.NSImage;
import dstep.appkit.NSTableView;
import dstep.applicationservices.coregraphics.CGBase;
import dstep.foundation.NSCoder;
import dstep.foundation.NSGeometry;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSSortDescriptor;
import dstep.foundation.NSString;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

enum
{
	NSTableColumnNoResizing = 0,
	NSTableColumnAutoresizingMask = ( 1 << 0 ),
	NSTableColumnUserResizingMask = ( 1 << 1 )
}

class NSTableColumn : NSObject, INSCoding
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}
	
	this (NSCoder aDecoder)
	{
		super(typeof(this).alloc.initWithCoder(aDecoder).objcObject);
	}
	
	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}
	
	typeof(this) initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(typeof(this), "initWithCoder:", NSCoder)(aDecoder);
	}

	NSTableColumn initWithIdentifier (Object identifier)
	{
		id result = invokeObjcSelf!(id, "initWithIdentifier:", Object)(identifier);
		return result is this.objcObject ? this : (result !is null ? new NSTableColumn(result) : null);
	}

	this (Object identifier)
	{
		super(NSTableColumn.alloc.initWithIdentifier(identifier).objcObject);
	}

	void setIdentifier (Object identifier)
	{
		return invokeObjcSelf!(void, "setIdentifier:", Object)(identifier);
	}

	Object identifier ()
	{
		return invokeObjcSelf!(Object, "identifier");
	}

	void setTableView (NSTableView tableView)
	{
		return invokeObjcSelf!(void, "setTableView:", NSTableView)(tableView);
	}

	NSTableView tableView ()
	{
		return invokeObjcSelf!(NSTableView, "tableView");
	}

	void setWidth (CGFloat width)
	{
		return invokeObjcSelf!(void, "setWidth:", CGFloat)(width);
	}

	CGFloat width ()
	{
		return invokeObjcSelf!(CGFloat, "width");
	}

	void setMinWidth (CGFloat minWidth)
	{
		return invokeObjcSelf!(void, "setMinWidth:", CGFloat)(minWidth);
	}

	CGFloat minWidth ()
	{
		return invokeObjcSelf!(CGFloat, "minWidth");
	}

	void setMaxWidth (CGFloat maxWidth)
	{
		return invokeObjcSelf!(void, "setMaxWidth:", CGFloat)(maxWidth);
	}

	CGFloat maxWidth ()
	{
		return invokeObjcSelf!(CGFloat, "maxWidth");
	}

	void setHeaderCell (NSCell cell)
	{
		return invokeObjcSelf!(void, "setHeaderCell:", NSCell)(cell);
	}

	Object headerCell ()
	{
		return invokeObjcSelf!(Object, "headerCell");
	}

	void setDataCell (NSCell cell)
	{
		return invokeObjcSelf!(void, "setDataCell:", NSCell)(cell);
	}

	Object dataCell ()
	{
		return invokeObjcSelf!(Object, "dataCell");
	}

	Object dataCellForRow (NSInteger row)
	{
		return invokeObjcSelf!(Object, "dataCellForRow:", NSInteger)(row);
	}

	void setEditable (bool flag)
	{
		return invokeObjcSelf!(void, "setEditable:", bool)(flag);
	}

	bool isEditable ()
	{
		return invokeObjcSelf!(bool, "isEditable");
	}

	void sizeToFit ()
	{
		return invokeObjcSelf!(void, "sizeToFit");
	}

	void setSortDescriptorPrototype (NSSortDescriptor sortDescriptor)
	{
		return invokeObjcSelf!(void, "setSortDescriptorPrototype:", NSSortDescriptor)(sortDescriptor);
	}

	NSSortDescriptor sortDescriptorPrototype ()
	{
		return invokeObjcSelf!(NSSortDescriptor, "sortDescriptorPrototype");
	}

	void setResizingMask (NSUInteger resizingMask)
	{
		return invokeObjcSelf!(void, "setResizingMask:", NSUInteger)(resizingMask);
	}

	NSUInteger resizingMask ()
	{
		return invokeObjcSelf!(NSUInteger, "resizingMask");
	}

	void setHeaderToolTip (NSString string)
	{
		return invokeObjcSelf!(void, "setHeaderToolTip:", NSString)(string);
	}

	NSString headerToolTip ()
	{
		return invokeObjcSelf!(NSString, "headerToolTip");
	}

	bool isHidden ()
	{
		return invokeObjcSelf!(bool, "isHidden");
	}

	void setHidden (bool hidden)
	{
		return invokeObjcSelf!(void, "setHidden:", bool)(hidden);
	}

	void setResizable (bool flag)
	{
		return invokeObjcSelf!(void, "setResizable:", bool)(flag);
	}

	bool isResizable ()
	{
		return invokeObjcSelf!(bool, "isResizable");
	}

}

