/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSTreeNode;

import dstep.foundation.NSArray;
import dstep.appkit.NSTreeController;
import dstep.foundation.NSIndexPath;
import dstep.foundation.NSObject;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

class NSTreeNode : NSObject
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}

	static Object treeNodeWithRepresentedObject (Object modelObject)
	{
		return invokeObjcSelfClass!(Object, "treeNodeWithRepresentedObject:", Object)(modelObject);
	}

	NSTreeNode initWithRepresentedObject (Object modelObject)
	{
		id result = invokeObjcSelf!(id, "initWithRepresentedObject:", Object)(modelObject);
		return result is this.objcObject ? this : (result !is null ? new NSTreeNode(result) : null);
	}

	this (Object modelObject)
	{
		super(NSTreeNode.alloc.initWithRepresentedObject(modelObject).objcObject);
	}

	Object representedObject ()
	{
		return invokeObjcSelf!(Object, "representedObject");
	}

	NSIndexPath indexPath ()
	{
		return invokeObjcSelf!(NSIndexPath, "indexPath");
	}

	bool isLeaf ()
	{
		return invokeObjcSelf!(bool, "isLeaf");
	}

	NSArray childNodes ()
	{
		return invokeObjcSelf!(NSArray, "childNodes");
	}

	NSMutableArray mutableChildNodes ()
	{
		return invokeObjcSelf!(NSMutableArray, "mutableChildNodes");
	}

	NSTreeNode descendantNodeAtIndexPath (NSIndexPath indexPath)
	{
		id result = invokeObjcSelf!(id, "descendantNodeAtIndexPath:", NSIndexPath)(indexPath);
		return result is this.objcObject ? this : (result !is null ? new NSTreeNode(result) : null);
	}

	NSTreeNode parentNode ()
	{
		id result = invokeObjcSelf!(id, "parentNode");
		return result is this.objcObject ? this : (result !is null ? new NSTreeNode(result) : null);
	}

	void sortWithSortDescriptors (NSArray sortDescriptors, bool recursively)
	{
		return invokeObjcSelf!(void, "sortWithSortDescriptors:recursively:", NSArray, bool)(sortDescriptors, recursively);
	}
}

