/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSURLResponse;

import dstep.foundation.NSCoder;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.foundation.NSURL;
import dstep.foundation.NSURLRequest;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

class NSHTTPURLResponse : NSURLResponse
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	NSInteger statusCode ()
	{
		return invokeObjcSelf!(NSInteger, "statusCode");
	}

	NSDictionary allHeaderFields ()
	{
		return invokeObjcSelf!(NSDictionary, "allHeaderFields");
	}

	static NSString localizedStringForStatusCode (NSInteger statusCode)
	{
		return invokeObjcSuperClass!(NSString, "localizedStringForStatusCode:", NSInteger)(statusCode);
	}
}

class NSURLResponse : NSObject, INSCoding, INSCopying
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	Object initWithURL (NSURL URL, NSString MIMEType, NSInteger length, NSString name)
	{
		return invokeObjcSelf!(Object, "initWithURL:MIMEType:expectedContentLength:textEncodingName:", NSURL, NSString, NSInteger, NSString)(URL, MIMEType, length, name);
	}

	this (NSURL URL, NSString MIMEType, NSInteger length, NSString name)
	{
		typeof(this).alloc.initWithURL(URL, MIMEType, length, name);
	}

	NSURL URL ()
	{
		return invokeObjcSelf!(NSURL, "URL");
	}

	NSString MIMEType ()
	{
		return invokeObjcSelf!(NSString, "MIMEType");
	}

	long expectedContentLength ()
	{
		return invokeObjcSelf!(long, "expectedContentLength");
	}

	NSString textEncodingName ()
	{
		return invokeObjcSelf!(NSString, "textEncodingName");
	}

	NSString suggestedFilename ()
	{
		return invokeObjcSelf!(NSString, "suggestedFilename");
	}

	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}

	Object initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(Object, "initWithCoder:", NSCoder)(aDecoder);
	}

	this (NSCoder aDecoder)
	{
		typeof(this).alloc.initWithCoder(aDecoder);
	}

	Object copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(Object, "copyWithZone:", NSZone*)(zone);
	}
}

